<?php
/**
 * Třída implementující metody pro práci s Adobe Connectem (7.5).
 */
class AcApi implements IAcApi, ILogable {

    //výchozí hodnoty pro uživatele
    private $userPassword = '1234'; //default user password for test
    private $sendEmail = 'true';

    //výchozí hodnoty pro místnosti
    private $type = 'meeting';
    private $userMeetings = 'user-meetings';

    //údaje pro přístup na test server
    //private $uri = 'http://tlupa3.cesnet.cz/api/xml';
    //private $username = 'testadmin';
    //private $password = 'Popocatepetl5';

    //udaje pro pristup na produkcni server s AC8
    private $uri = 'https://connect.cesnet.cz/api/xml';
    private $username = 'peruntester';
    private $password = 'slo6it0_Hesl94ko';
    private $salt = '_nejaky_nahodny_retezec_';


    private $_client; // AcApi_Client

    /**
     * Konstruktor zajišťující připojení se na server Adobe Connectu.
     */
    function  __construct() {

// Client configuration
        $acConfig = array(
                // Type of transport to be used
                'type' => 'xml',
                // The URL of the remote AC API
                'uri' => $this->uri,
                // The user for the connection
                'username' => $this->username,
                // The user's password
                'password' => $this->password,
                // The name of the cookie holding the current session
                'cookie' => 'BREEZESESSION',
                // Set logging to standard output, could be a file too
                'log' => 'php://stderr'//output'
        );

// Client initialization
        $this->_client = new AcApi_Client($acConfig);
// Login to the server API
        $resp = $this->_client->login();
        $this->logDebug('Logging into AdobeConnect...', '__construct');
        if ($resp->isError()) {
            $this->logError($resp, '__construct');
        }
    }

    /**
     * Vytvoří uživatele v Adobe Connect.
     * @param <type> $user instance User
     * @return principalId nově vytvořeného uživatele, nebo -1 v případě chyby.
     */
    public function createUser($user) {
        $result = -1;
        $methodName = 'createUser';
        $this->logDebug('Entering method '.$methodName, $methodName);
        $principalId = $this->getPrincipalId($user);
        if ($principalId > 0) { //user already exists in AC..
            return $principalId;
        }
        $this->logDebug('Creating user: ['.$user->toString().']', $methodName);
        $resp = $this->_client->api_principalUpdate(array(
                'email' => $user->getEmail(),
                'first-name' => $user->getName() , 'has-children' => 'false','last-name' => $user->getLastname(),
                'login' => $user->getEppn(), 'password' => md5($user->getEppn().$this->salt),'send-email' => $this->sendEmail, 'type' => 'user'
        ));

        $xml = new SimpleXMLElement($resp->getXml());
        if ($this->isStatusCodeOk($xml) ) {
            $result = $xml->principal['principal-id'];
        } else {
            $this->logError($resp, $methodName);
        }
        $this->logDebug('principalId of new user: ['.$result.']', $methodName);
        // vysledky si loguje a zpracovava metoda ktera jej vola
        return $result;
    }

    /**
     *
     *  Smaže uživatele z AdobeConnectu
     * @param <type> $user instance User
     * @return boolean:
     * false při chybě
     * true při úšpěšném smazání
     * true pokud již uživatel v systému neexistuje (není co smazat)
     */
    public function deleteUser($user) {
        $methodName = 'deleteUser';
        $this->logDebug('Entering method '.$methodName, $methodName);

        $principalId = $this->getPrincipalId($user);
        if ($principalId == -1) {
            return false; // error
        }
        if ($principalId == -10) {
            return true; //user already does not exists in AC
        }
        $this->logDebug('Deleting user: ['.$user->toString().']', $methodName);
        $resp = $this->_client->api_principalsDelete(array(
                'principal-id' => $principalId//find principal id of user
        ));

        $xml = new SimpleXMLElement($resp->getXml());
        if (! $this->isStatusCodeOk($xml)) {
            //$this->processError($resp, 'deleteUser');
            $this->logError('Deleting user failed. Reponse: '.$resp, $methodName);
        }
        $this->logDebug('User ['.$user->toString().'] deleted succesfully.', $methodName);
        return true;

    }

    /**
     * Vytvoří konferenční místnost v Adobe Connect.
     * @param <type> $room instance Room
     * @return int:
     * -1 při chybě
     * -10 pokud již místnost existuje v Adobe Connect (nepřepisuje se)
     * kladné číslo při úspěchu (reprezentuje scoId nově vytvořené místnosti)
     */
    public function createRoom($room) {
        $result = -1; //default error state
        $methodName = 'createRoom';
        $this->logDebug('Entering method '.$methodName, $methodName);

        $folderId = $this->getFolderId();
        if ($folderId < 0) {
            return $result; //return error code, can't create room due to missing folderId
        }

        $scoId = $this->getScoId($room);
        if ($scoId > 0) { //room with this name already exists
            return $scoId;
        }

        $this->logDebug('Creating room: ['.$room->toString().']', $methodName);
        $resp = $this->_client->api_scoUpdate(array(
                'folder-id' => $folderId,
                'name' => $room->getName(),
                'type' => $this->type //type is meeting, see manual page 235
        ));

        $xml = new SimpleXMLElement($resp->getXml());
        if ($this->isStatusCodeOk($xml)) {
            $result = $xml->sco['sco-id'];
        } else if ($xml->status['code'] == 'invalid' && $xml->status->invalid['subcode'] == 'duplicate') {
            $result = -10; //meeting room already exists in AC
            $this->logDebug('Room ['.$room->toString().'] already exists in AC.', $methodName);
        } else {
            $this->logError($resp, $methodName);
        }
        $this->logDebug('ScoId of created room: ['.$result.']', $methodName);
        return $result;
    }

    /**
     * Smaže místnost z AC
     * @param <type> $room instance Room
     * @return boolean:
     * true při úspěšném smazání
     * true pokud již místnost v AC neexistuje
     * false při chybě
     */
    public function deleteRoom($room) {

        $methodName = 'deleteRoom';
        $this->logDebug('Entering method '.$methodName, $methodName);

        $scoId = $this->getScoId($room);
        $this->logDebug('ScoId of room: ['.$scoId.']', $methodName);
        if ($scoId < 0 && $scoId != -20) { //error
            return false;
        }
        if ($scoId == -20) {
            return true; //room already does not exists in AC, ok
        }

        $methodName = 'deleteRoom';
        $this->logDebug('Deleting room: ['.$room->toString().']', $methodName);
        $resp = $this->_client->api_scoDelete(array(
                'sco-id' => $scoId,
        ));
        $xml = new SimpleXMLElement($resp->getXml());
        if ($this->isStatusCodeOk($xml)) {
            $this->logDebug('Room ['.$room->toString().'] succesfully deleted.', $methodName);
            return true;
        } else {
            $this->logError($resp, $methodName);
            return false; //error response from AC
        }
    }

    /**
     * Upraví uživateli přístupové práva do místnosti.
     * @param <User> $user
     * @param <Room> $room
     * @param <Permission> $permission
     * @return <boolean> :
     * true při úspěchu
     * false při chybě
     */
    public function updateUserToRoom($user, $room, $permission) {
        $scoId = $this->getScoId($room);
        $principalId = $this->getPrincipalId($user);
        $result = false; //default error
        $methodName = 'updateUserToRoom';
        $this->logDebug('Entering method '.$methodName, $methodName);

        if ($scoId > 0 && $principalId > 0 && !empty($permission)) {
            $this->logDebug('Updating permission to user: ['.$user->toString().'] room: ['.$room->toString().'] permission: ['.$permission.'] acl-id = '.$scoId.' principalId ='.$principalId.' permissionId = '.$permission, $methodName);
            $resp = $this->_client->api_permissionsUpdate(array(
                    'acl-id' => $scoId,
                    'principal-id' => $principalId,
                    'permission-id' => $permission
            ));
            $xml = new SimpleXMLElement($resp->getXml());
            if ($this->isStatusCodeOk($xml)) {
                $result = true;
                $this->logDebug('Update of permission is done sucesfully.', $methodName);
            } else {
                $this->logError($resp, $methodName);
                $result = false; //error response from AC
            }
        }

        return $result;
    }

    // private methods ----------------------------------------------------

    /**
     * Vrátí interní AC principalId (= id uživatele)
     * @param <User> $user - musí mít vyplněno eppn
     * @return <int>
     * -1: chyba
     * -10  - uživatel neexistuje
     * kladné číslo: nalezené principalId uživatele
     */
    private function getPrincipalId($user) {
        $result = -1; // -1 means error
        $methodName = 'getPrincipalId';
        $this->logDebug('Entering method '.$methodName, $methodName);

        $eppn = $user->getEppn();
        if (empty($eppn)) {
            $this->logError('Missing eppn '.$user->toString(), $methodName);
        } else {

            $this->logDebug('Looking for user: ['.$user->toString().']', $methodName);
            $resp = $this->_client->api_principalList(array(
                    'filter-login' => $user->getEppn()
            ));

            $xml = new SimpleXMLElement($resp->getXml());
            if ($this->isStatusCodeOk($xml) ||$xml->status['code'] == 'no-data') {
                $principalId = $xml->{'principal-list'}->principal['principal-id'];
                if (!empty($principalId)) { //if user exists in AC...
                    $result = $principalId;
                } else {
                    $this->logDebug('The user probably does not exist any more in AC.', $methodName);
                    $result = -10; //indicates that user does not exist
                }
            } else {
                $this->logError($resp, $methodName);
            }
        }
        $this->logDebug('Returning principalId: ['.$result.'] of user: ['.$user->toString().']', $methodName);
        return $result;
    }

    //see manual page 38 for retrieve sco-id aka folder id and convert it to php!
    /**
     * Získá interní AC folderId uživatele, pod kterým pracuje tento skript.
     * Toto folderId reprezentuje jeho složku, kde může vytvářet místnosti.
     * @return <int>
     * kladné číslo: značí úspěch a vrací folderId
     * -1: chyba
     */
    private function getFolderId() {
        $methodName = 'getFolderId';
        $this->logDebug('Entering method '.$methodName, $methodName);

        $resp = $this->_client->api_scoShortcuts(array());
        //retrieve sco-id from response which represents user's (his meetings) folder-id
        $xmlElement = $resp->getXml();
        $xml = new SimpleXMLElement($xmlElement);

        if ($this->isStatusCodeOk($xml)) {
            foreach ($xml->shortcuts[0]->sco as $sco) {
                //we want sco-id of type user-meetings
                if ($sco['type']== $this->userMeetings) {
                    $scoId = $sco['sco-id'];
                    $this->logDebug('Returning scoId/folderId: ['.$scoId.']', $methodName);
                    return $scoId; //return sco-id of the meeting = folderId
                }
            }
        }
        //no folderId found or statusCode is not ok, so log error and return error state
            $this->logError($resp, $methodName);
            return -1; //if we dont get sco-id...
    }

    /**
     * Vrátí interní scoId (id místnosti) AC pro danou místnost.
     * @param <Room> $room
     * @return <int>
     * -1 : chyba
     * -20 : nic nenalezeno
     * kladné číslo: vrací nalezené scoId
     */
    private function getScoId($room) {
        $result = -1; //default error code
        $methodName = 'getScoId';
        $this->logDebug('Entering method '.$methodName, $methodName);

        $name = $room->getName();
        if (empty($name)) {
            $this->logError('Name of room is required ['.$room->toString().']', $methodName);
        }

        $this->logDebug($room->toString(), $methodName);
        $resp = $this->_client->api_scoSearchByField(array(
                'query' => $name,
                'field' => $name
        ));
        //retrieve sco-id from response which represents user's (his meetings) folder-id
        $xmlElement = $resp->getXml();
        $xml = new SimpleXMLElement($xmlElement);

        if ($this->isStatusCodeOk($xml)) { //AC returns ok even if no data found
            $scoId = $xml->{'sco-search-by-field-info'}->sco['sco-id'];
            ($scoId > 0 ? $result = $scoId : $result = -20); //-20 means nothing found
        } else {
            $this->logError($resp, $methodName);
        }
        $this->logDebug('Returning result scoId: ['.$result.']', $methodName);
        return $result;
    }

    public function logDebug($description, $methodName) {
        SysLogger::logDebug($description, 'AcApi', $methodName);
    }

    public function logError($description, $methodName) {
        SysLogger::logError($description, 'AcApi', $methodName);
    }

    /**
     * Kontroluje xml element odpovědi z AC zda-li byl dotaz úspěšný.
     * Všimni si: i při správném dotazu může být odpověd např. status-code=no-data atd.
     * tzn. tato metoda se nehodí pro obecné ověřování na status-code=invalid
     * a není doporučeno ji pro tento účel používat.
     * @param <type> $xml
     * @return <boolean>
     * true pokud je status-code=ok
     * false jinak
     */
    private function isStatusCodeOk($xml) {
        return $xml->status['code'] == 'ok';
    }
}
?>
