<?php

/**
 * Tato trida poskytuje verejne staticke metody pro logovani.
 * K logovani je vyuzita sluzba syslog. Zpravy jak s oznacenim Debug, tak i
 * Error jsou na openSuse zalogovany do souboru /var/log/messages
 * Každá logová zpráva obsahuje defaultně slovo AcDaemonService.
 *
 * Author: xharazim
 */
class SysLogger {

    //to se objeví na každém řádku v syslogu
    private static $mark = 'AcDaemonService';
    private $printOnDashboard = true;

    /**
     * Slouží k inicializaci logování. Volá se sama ihned po třídě, není ji
     * tedy třeba volat explicitně. Nastavuje klíčové slovo (mark).
     */
    public static function init() {
        openlog(SysLogger::$mark, LOG_PERROR, LOG_USER);
    }

    /**
     * Zaloguje do syslogu hlášku s označením [DEBUG].
     * @param <type> $description - zpráva o chybě
     * @param <type> $class - jméno třídy odkud zpráva jde
     * @param <type> $methodName - jméno odkud zpráva jde
     */
    public static function logDebug($description, $class, $methodName) {
        if ($this->printOnDashboard) {
            echo(' [DEBUG] '.$class.'.'.$methodName.'() description: '.$description."<br>");
        }
        if (!syslog(LOG_DEBUG, ' [DEBUG] '.$class.'.'.$methodName.'() description: '.$description)) {
            $this->logError($description, $class, $methodName);
        }

    }

    /**
     * Zaloguje do syslogu hlášku s označením [ERROR]
     * Po zalogování zprávy dojde k vyvolání metody processError().
     * ToDo predat volani processError() za logovani v klientskych tridach
     * @param <type> $description
     * @param <type> $class
     * @param <type> $methodName
     */
    public static function logError($description, $class, $methodName) {

        $time = time();
        $logMessage = ' [ERROR] id: '.$time.' In: '.$class.'.'.$methodName.'() description: '.$description;
        if ($this->printOnDashboard) {
            echo ($logMessage."<br>");
        }
        if (!syslog(LOG_DEBUG, $logMessage)) {
            SysLogger::processError('Using syslog() failed. Root cause: '.$description, 'SysLogger','logError');
        }
        //process error (i.e. send mail to admin)
        SysLogger::processError($logMessage, 'SysLogger','logError');
        //stop the script - throw exception...
        throw new Exception('[ERROR] An error occured. Script stopped. Cause: '.$logMessage);
    }

    /**
     * Zpracuje chybu.
     * Odešle informační mail správci systému.
     * @param <type> $description
     * @param <type> $class
     * @param <type> $methodName
     */
    public static function processError($description, $class, $methodName) {
        //ToDo send Mail
        $mailer = new Mailer();
        $mailer->sendMail($description);

    }



}
//náhrada za statický inicializátor, volá se automaticky
SysLogger::init();

?>
