#!/bin/bash

PROTOCOL_VERSION='3.0.0'


function process {
  DST_FILE="/etc/group-nfs4"

  ### Status codes
  I_CHANGED=(0 "${DST_FILE} updated")
  I_NOT_CHANGED=(0 "${DST_FILE} has not changed")

  E_GROUP_FILTER=(50 'Error in /etc/group filter')
  E_GROUP_WRONG_MIN_GID=(51 'Invalid min_gid parameter')
  E_GROUP_WRONG_MAX_GID=(52 'Invalid max_gid parameter')
  E_GROUP_MERGE=(53 'Error during group file merge')
  E_GROUP_DUPLICATES=(54 'Group names in group file are not uniq: ${DUPLICATE_GROUP_NAMES}')
  E_GROUP_NO_ROOT=(55 'Missing group "root"')

  FROM_PERUN="${WORK_DIR}/group_nfs4"

  OLD_GROUP="$DST_FILE"
  #NEW_GROUP="/etc/passwd.new" # file must be on same mountpoint for atomic switch
  NEW_GROUP="${WORK_DIR}/group.new"
  MIN_PERUN_GID=`head -n 1 "${WORK_DIR}/min_gid"`
  MAX_PERUN_GID=`head -n 1 "${WORK_DIR}/max_gid"`
  [ "${MIN_PERUN_GID}" -gt 0 ] || log_msg E_GROUP_WRONG_MIN_GID
  [ "${MAX_PERUN_GID}" -gt 0 ] || log_msg E_GROUP_WRONG_MAX_GID

  create_lock

  #if OLD_GROUP doesn't exist, create it
  [ -e "$OLD_GROUP" ] || touch "$OLD_GROUP"

  # grep lines from /etc/group which gid is NOT between min a max gid
  catch_error E_GROUP_FILTER perl -F':' -ane "print if(\$F[2] < $MIN_PERUN_GID || \$F[2] > $MAX_PERUN_GID)" "${OLD_GROUP}" >"${NEW_GROUP}"

  # merge received group into striped old one                                                    
  catch_error E_GROUP_MERGE cat "${FROM_PERUN}" >>"${NEW_GROUP}"

  # check for duplicate group names in /etc/group                                                       
  DUPLICATE_GROUP_NAMES=`cut -d: -f1 "${NEW_GROUP}" | sort | uniq -d`
  [ "x${DUPLICATE_GROUP_NAMES}" == 'x' ] || log_msg E_GROUP_DUPLICATES

  # move only in case there are any changes
  diff_mv "${NEW_GROUP}" "${DST_FILE}" \
          && log_msg I_CHANGED \
          || log_msg I_NOT_CHANGED
}
