#!/bin/bash

PROTOCOL_VERSION='3.0.0'


function process {
  ### Status codes
  I_CHANGED=(0 "${DST_FILE} updated")
  I_NOT_CHANGED=(0 "${DST_FILE} has not changed")

  ### Error codes ###
	E_VOMS_DELETE=(50 'Error during VOMS delete-user operation')
	E_VOMS_CREATE=(50 'Error during VOMS create-user operation')
	E_VOMS_GET_USERS=(50 'Error during getting VOMS users for VO ${VO}')

  FROM_PERUN="${WORK_DIR}/voms"

  create_lock

	# Get list of VOs
  VOS=`cat ${FROM_PERUN} | sed 's/^\([[:alnum:]_-]*\):.*/\1/' | uniq`

  # Iterate through every VO and check which user will be added or removed
  echo -e "$VOS" | while read VO; do
        # Get users from the VO
        VO_USERS=`cat ${FROM_PERUN} | grep "^${VO}:.*" | sort`
        # Get current users stored in VOMS and convert lines into PERUN format
        catch_error E_VOMS_GET_USERS CURRENT_VO_USERS=`voms-admin --vo "${VO}" list-users | sed "s/\(.*\), \(.*\) - \(.*\)/${VO}:\1:\2::\3/" | sort`
				

        # Check who should be deleted
        echo -e "$CURRENT_VO_USERS" | while read CURRENT_VO_USER; do
                if [ `echo $VO_USERS | grep -c "$CURRENT_VO_USER"` -eq 0 ]; then
                        # User is not in VO anymore, so remove him      
                        echo $CURRENT_VO_USER | while IFS=":" read VO_SHORTNAME USER_DN CA_DN USER_NAME USER_EMAIL; do
                                catch_error E_VOMS_DELETE voms-admin --nousercert --vo "\"${VO_SHORTNAME}\"" delete-user "\"$USER_DN\"" "\"$CA_DN\""
                        done


                fi
        done
        # Check who should be added
        echo -e "$VO_USERS" | while read VO_USER; do
                if  [ `echo $CURRENT_VO_USERS | grep -c "$VO_USER"` -eq 0 ]; then
                        # New user comming, so add him to the VO
                        echo $VO_USER | while IFS=":" read VO_SHORTNAME USER_DN CA_DN USER_NAME USER_EMAIL; do
                                USER_CN=`echo $USER_DN | sed 's/.*CN=\(.*\)/\1/'`
                                catch_error E_VOMS_CREATE voms-admin --nousercert --vo "\"${VO_SHORTNAME}\"" create-user "\"$USER_DN\"" "\"$CA_DN\"" "\"$USER_CN\"" "\"$USER_EMAIL\""
                        done
                fi
        done
  done
}
